﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

using System.Runtime.InteropServices; // needed for DllImport
using System.Text; // needed for StringBuilder Class
using System.IO;

// Please read the included file "important_notes_for_NET_developers_on_using_WSQ_image_library.html"
// or the page: 
// http://www.cognaxon.com/index.php?page=wsqlib_NETnotes
// FAQ (Frequently Asked Questions)
// for .NET developers on using WSQ image library



namespace NIST_Type_8_Microsoft_CSharp
{
    public partial class Form1 : Form
    {
        private WSQImageLibraryNativeMethods WSQ_library_native_methods;

        private IntPtr bp;
        private System.Drawing.Bitmap b;
        private byte[] input_data_stream;
//        private byte[] input_data_stream2;
//        private byte[] output_data_stream;
//        private byte[] output_data_stream2;


        private Int32 type8_image_compression_type;
        private String result_string;
        private String save_type8_binary_field_filename;
        private String image_size_txt_file;

        private IntPtr type8_binary_field_ptr;
        private byte[] type8_binary_field;
        private Int32 type8_binary_field_length;


        private Int64 type8_image_width;
        private Int64 type8_image_height;
		private UTF8Encoding utf8 = new UTF8Encoding();



        public Form1()
        {
            InitializeComponent();

            type8_binary_field_ptr = IntPtr.Zero;

            comboBoxCompression.Items.Add("Uncompressed");
            comboBoxCompression.Items.Add("ANSI/EIA-538-1988 facsimile compression");
            comboBoxCompression.SelectedIndex = 0;

            textBoxWidth.Text = "0";
            textBoxHeight.Text = "0";

            WSQ_library_native_methods = new WSQImageLibraryNativeMethods();

        }


        // To call functions from WSQ_library64.dll
		// you can use the methods from wrapper class WSQImageLibraryNativeMethods
		// or you can use function prototypes provided below 

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr CreateBMPFromFile(byte[] filename);

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern void SaveBMPToFile(IntPtr hBitmap, byte[] filename, Int64 filetype);

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		public static extern void RegisterWSQ();

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		public static extern void SetShowFilePropertiesDialog(Int64 file_properties_dialog);

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		public static extern void ShowFileConverter();

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		public static extern IntPtr ConvertHBITMAPtoGrayScale256(IntPtr hBitmap);

		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern void SaveHBITMAPtoFileAsGrayScale256BMP(IntPtr hBitmap, byte[] filename);
		

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr CreateNISTType8BinaryFieldfromHBITMAP(IntPtr input_HBITMAP, int type8_binary_field_compression_type);

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr CreateHBITMAPfromNISTType8BinaryField(Int64 type8_image_width, Int64 type8_image_height, Int64 type8_compression_type, [In] byte[] type8_input_byte_array, Int64 type8_input_byte_array_size);

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern Int64 ReadSizeOfNISTType8BinaryField();

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern Int64 ReadType8DataCompressionType();

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern Int64 ReadType8DataWidth();

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern Int64 ReadType8DataHeight();

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern Int64 SetNISTType8ImageColorInversion(Int64 in_uncompressed_type8_image_inverted, Int64 out_uncompressed_type8_image_inverted, Int64 in_compressed_type8_image_inverted, Int64 out_compressed_type8_image_inverted);


        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GenerateWSQLibrarySerialNumber();

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern Int32 UnlockWSQLibrary(byte[] authorization_code);





        private void ConvertFromType8Button_Click(object sender, EventArgs e)
        {


            ConvertToType8Button.Enabled = false;
            ConvertFromType8Button.Enabled = false;
            buttonOpen.Enabled = false;
            buttonSave.Enabled = false;


            OpenFileDialog ofd = new OpenFileDialog();

            ofd.Filter = "Type8 binary images|*.bin";

            ofd.FilterIndex = 1;
            if (ofd.ShowDialog() != DialogResult.OK)
            {
                ConvertToType8Button.Enabled = true;
                ConvertFromType8Button.Enabled = true;
                return;
            }


            image_size_txt_file = Path.ChangeExtension(ofd.FileName, ".txt");

            if (File.Exists(image_size_txt_file))
            {

                // Create an instance of StreamReader to read from a file.
                // The using statement also closes the StreamReader.
                using (StreamReader sr = new StreamReader(@image_size_txt_file))
                {
                    type8_image_compression_type = Convert.ToInt32(sr.ReadLine());  // type8_image_compression_type = int.Parse(sr.ReadLine())
                    type8_image_width = Convert.ToInt32(sr.ReadLine());
                    type8_image_height = Convert.ToInt32(sr.ReadLine());

                    textBoxWidth.Text = Convert.ToString(type8_image_width);
                    textBoxHeight.Text = Convert.ToString(type8_image_height);

                }
            }//if (File.Exists(image_size_txt_file)) 
            else
            {
                type8_image_width = Convert.ToInt32(textBoxWidth.Text);
                type8_image_height = Convert.ToInt32(textBoxHeight.Text);            
            }


   if((type8_image_width == 0) | (type8_image_height == 0))
   {
    MessageBox.Show("You must manually set the type8_image_width and type8_image_height and compression type of the image.");
    ConvertFromType8Button.Enabled = true;
    buttonOpen.Enabled = true;
    return;
   }


            comboBoxCompression.SelectedIndex = type8_image_compression_type;


            FileInfo fInfo = new FileInfo(ofd.FileName);
            long input_stream_length = fInfo.Length;
            FileStream fStream = new FileStream(ofd.FileName, FileMode.Open, FileAccess.Read);
            BinaryReader br = new BinaryReader(fStream);

            input_data_stream = br.ReadBytes((int)input_stream_length);
            br.Close();
            fStream.Close();


            bp = CreateHBITMAPfromNISTType8BinaryField(type8_image_width, type8_image_height, type8_image_compression_type, input_data_stream, (int)input_stream_length);

            b = Image.FromHbitmap(bp);
            picture.Image = b;
            
            Int32 h = picture.Image.Height;
            Int32 w = picture.Image.Width;
            picture.SetBounds(0, 0, w, h);


            ConvertToType8Button.Enabled = true;
            ConvertFromType8Button.Enabled = true;
            buttonOpen.Enabled = true;
            buttonSave.Enabled = true;

        }

        private void ConvertToType8Button_Click(object sender, EventArgs e)
        {

            ConvertToType8Button.Enabled = false;
            ConvertFromType8Button.Enabled = false;


            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "BIN Files (*.bin)|*.bin";
            sfd.FilterIndex = 1;
            if (sfd.ShowDialog() != DialogResult.OK)
            {
                ConvertFromType8Button.Enabled = true;
                ConvertFromType8Button.Enabled = true;
                return;
            }


            type8_image_width = Convert.ToInt32(textBoxWidth.Text);
            type8_image_height = Convert.ToInt32(textBoxHeight.Text);
            type8_image_compression_type = comboBoxCompression.SelectedIndex;

            image_size_txt_file = Path.ChangeExtension(sfd.FileName, ".txt");


            using (System.IO.StreamWriter file = new System.IO.StreamWriter(@image_size_txt_file))
            {
                file.WriteLine(type8_image_compression_type.ToString());
                file.WriteLine(type8_image_width.ToString());
                file.WriteLine(type8_image_height.ToString());
            }


            b = new Bitmap(picture.Image);
            bp = b.GetHbitmap();

            save_type8_binary_field_filename = Path.ChangeExtension(sfd.FileName, ".bin");

            type8_binary_field_ptr = CreateNISTType8BinaryFieldfromHBITMAP(bp, type8_image_compression_type);
	        type8_binary_field_length = (Int32)ReadSizeOfNISTType8BinaryField();

            //MessageBox.Show(type8_binary_field_length.ToString());

	        type8_binary_field = new byte[type8_binary_field_length];
	        Marshal.Copy(type8_binary_field_ptr, type8_binary_field, 0, type8_binary_field_length);

            FileStream fStream = new FileStream(save_type8_binary_field_filename, FileMode.Create);
	        BinaryWriter bw = new BinaryWriter(fStream);

	        bw.Write(type8_binary_field);
	        bw.Close();
	        fStream.Close();
           
            
            result_string = "The converted file was saved as:\n" + save_type8_binary_field_filename;
	        MessageBox.Show(result_string);

            ConvertToType8Button.Enabled = true;
            ConvertFromType8Button.Enabled = true;

        }

        private void buttonOpen_Click(object sender, EventArgs e)
        {

            OpenFileDialog ofd = new OpenFileDialog();

            ofd.Filter = "All images|*.wsq;*.bmp;*.tiff;*.tif;*.jpg;*.jpeg;*.jp2;*.jpc;" +
"*.png;*.gif;*.tga;*.pcx;*.ppm;*.pgm;*.pbm;*.ico;*.wmf;*.wbmp;*.ska;*.mng;*.pnm;*.ras;*.raw;|" +
"FBI's Wavelet Scalar Quantization (*.wsq)|*.wsq|" +
"Windows Bitmap Graphics (*.bmp)|*.bmp|" +
"Macintosh Tagged Information File Format images (*.tiff)|*.tiff|" +
"PC Tagged Information File Format images (*.tif)|*.tif|" +
"Joint Photographic Experts Group images (*.jpg)|*.jpg|" +
"Joint Photographic Experts Group images (*.jpeg)|*.jpeg|" +
"JPEG2000 Part 1 images (*.jp2)|*.jp2|" +
"JPEG2000 Code Stream images (*.jpc)|*.jpc|" +
"Portable Network Graphics images (*.png)|*.png|" +
"Compuserve Graphic Interchange Format images (*.gif)|*.gif|" +
"Truevision Targa Graphic images (*.tga )|*.tga|" +
"ZSoft Paintbrush images (PC Paintbrush Bitmap Graphic) (*.pcx)|*.pcx|" +
"Portable Pixel Map Images (*.ppm)|*.ppm|" +
"Portable Gray Map Images (*.pgm)|*.pgm|" +
"Portable Bitmap Images (*.pbm)|*.pbm|" +
"Windows Icons (*.ico)|*.ico|" +
"Windows Metafiles (*.wmf)|*.wmf|" +
"Wireless Bitmap File Format (*.wbmp)|*.wbmp|" +
"SKA Format images (*.ska)|*.ska|" +
"Muti-image Network Graphic Animation (*.mng)|*.mng|" +
"PBM Portable Any Map Graphic Bitmap (*.pnm)|*.pnm|" +
"Sun Graphic (*.ras)|*.ras|" +
"RAW Graphic (*.raw)|*.raw";

            ofd.FilterIndex = 1;
            if (ofd.ShowDialog() != DialogResult.OK)
            {
                return;
            }


            //you can use commented line of code bellow or you can use method from class WSQ_library_native_methods
            //bp = CreateBMPFromFile(utf8.GetBytes(ofd.FileName));
            bp = WSQ_library_native_methods.pCreateBMPFromFile(ofd.FileName);
            b = Image.FromHbitmap(bp);
            picture.Image = b;

            Int32 h = picture.Image.Height;
            Int32 w = picture.Image.Width;
            picture.SetBounds(0, 0, w, h);

            buttonSave.Enabled = true;
            ConvertToType8Button.Enabled = true;


        }

        private void buttonSave_Click(object sender, EventArgs e)
        {

			SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "WSQ Files (*.wsq)|*.wsq|" + // 1
"Windows BMP (*.bmp)|*.bmp|" + // 2
"TIFF Files (*.tif)|*.tif|" + // 3
"PNG Files (*.png)|*.png|" + // 4
"JPEG Files (*.jpg)|*.jpg|" + // 5
"Targa Files (*.tga)|*.tga|" + // 6
"SGI Files (*.rgb)|*.rgb|" + // 7
"JPEG-2000 Part-1 (*.jp2)|*.jp2|" + // 8
"JPEG-2000 Code Stream (*.jpc)|*.jpc|" + // 9
"PBM - Portable Bitmap Format (*.pbm)|*.pbm|" + // 10
"PGM - Portable Graymap Format (*.pgm)|*.pgm|" + // 11
"PPM - Portable Pixmap Format (*.ppm)|*.ppm|" + // 12
"GIF Files (*.gif)|*.gif|" + // 13
"PNM Files (*.pnm)|*.pnm|" + // 14
"RAS Files (*.ras)|*.ras|" + // 15
"MNG Files (*.mng)|*.mng|" + // 16
"PCX Files (*.pcx)|*.pcx"; // 17

            sfd.FilterIndex = 1;
            if (sfd.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            //you can use commented line of code bellow or you can use method from class WSQ_library_native_methods
            //SetShowFilePropertiesDialog(1);
            WSQ_library_native_methods.pSetShowFilePropertiesDialog(1);


            b = new Bitmap(picture.Image);
            bp = b.GetHbitmap();

            //you can use commented line of code bellow or you can use method from class WSQ_library_native_methods
            //SaveBMPToFile(bp, utf8.GetBytes(sfd.FileName), sfd.FilterIndex);
            WSQ_library_native_methods.pSaveBMPToFile(bp, sfd.FileName, sfd.FilterIndex);

        }
    }
}
