using System;
using System.Collections;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

// Important notice:
// in file "WSQImageLibraryNativeMethods.cs"
// "namespace" must match the "namespace" of the main project
namespace NIST_Type_8_Microsoft_CSharp
{
    class WSQImageLibraryNativeMethods
    {

        static IntPtr wsq_decompressed_byte_array_ptr = IntPtr.Zero;
		static IntPtr out_comment_text_ptr = IntPtr.Zero;
		static IntPtr wsq_compressed_byte_array_ptr = IntPtr.Zero;
		static IntPtr bitmap_ptr = IntPtr.Zero;
        IntPtr serial_number_ptr = IntPtr.Zero;

        private UTF8Encoding utf8 = new UTF8Encoding();



        public WSQImageLibraryNativeMethods() { }


        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr GenerateWSQLibrarySerialNumber();
        public IntPtr pGenerateWSQLibrarySerialNumber()
        {
            try
            {
                serial_number_ptr = GenerateWSQLibrarySerialNumber();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GenerateWSQLibrarySerialNumber Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return serial_number_ptr;
        }


        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern Int64 UnlockWSQLibrary(byte[] unlock_code);
        public Int64 pUnlockWSQLibrary(String unlock_code_string)
        {
            Int64 retVal = -1;

            try
            {
                retVal = UnlockWSQLibrary(utf8.GetBytes(unlock_code_string));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "UnlockWSQLibrary Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return retVal;
        }


        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		private static extern void RegisterWSQ();
		public void pRegisterWSQ()
		{
			try
			{
				RegisterWSQ();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "RegisterWSQ Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr CreateBMPFromFile(byte[] lpszFileName);
        public IntPtr pCreateBMPFromFile(String FileName)
        {

            if (FileName == null)
                throw new ArgumentNullException("FileName should be not null");
            if (FileName.Length == 0)
                throw new ArgumentException("Incorrect file name");
            try
            {
                bitmap_ptr = CreateBMPFromFile(utf8.GetBytes(FileName));
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message, "CreateBMPFromFile Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return bitmap_ptr;
        }

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern Int64 SaveBMPToFile(IntPtr hBitmap, byte[] filename,
            [MarshalAs(UnmanagedType.I8)] Int64 filetype);
        public Int64 pSaveBMPToFile(IntPtr Bitmap, String FileName, Int64 FileType)
        {
            Int64 retVal = -1;
            if ((Bitmap == (IntPtr)null)||(FileName == null)) 
                throw new ArgumentNullException("Bitmap and FileName should be not null");
            if (FileName.Length == 0)
                throw new ArgumentException("Incorrect file name");
            try
            {
                retVal = SaveBMPToFile(Bitmap, utf8.GetBytes(FileName), FileType);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "SaveBMPToFile Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return retVal;
        }


		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		private static extern IntPtr CreateBMPFromWSQByteArray(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] [MarshalAs(UnmanagedType.I8)] Int64 input_wsq_compressed_byte_array_length);
		public IntPtr pCreateBMPFromWSQByteArray(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] Int64 input_wsq_compressed_byte_array_length)
		{
            
			try
			{
				bitmap_ptr = CreateBMPFromWSQByteArray(input_wsq_compressed_byte_array, input_wsq_compressed_byte_array_length);                                     
				}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "CreateBMPFromWSQByteArray Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			return bitmap_ptr;
		}


		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		private static extern Int64 SaveWSQByteArrayToImageFile(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] [MarshalAs(UnmanagedType.I8)] Int64 input_wsq_compressed_byte_array_length,
			byte[] filename,
			[MarshalAs(UnmanagedType.I8)] Int64 filetype);
		public Int64 pSaveWSQByteArrayToImageFile(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] Int64 input_wsq_compressed_byte_array_length,
			String FileName,
			Int64 FileType)
		{
            Int64 retVal = -1;
			if (FileName == null)
									 throw new ArgumentNullException("FileName should be not null");
			if (FileName.Length == 0)
				throw new ArgumentException("Incorrect file name");

			try
			{
				retVal = SaveWSQByteArrayToImageFile(input_wsq_compressed_byte_array, input_wsq_compressed_byte_array_length,
					utf8.GetBytes(FileName), FileType);                                        
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "SaveWSQByteArrayToImageFile Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			return retVal;
		}





        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern void SetShowFilePropertiesDialog(
               [MarshalAs(UnmanagedType.I8)] Int64 file_properties_dialog);
        public void pSetShowFilePropertiesDialog(Int64 file_properties_dialog)
        {
            try
            {
                SetShowFilePropertiesDialog(file_properties_dialog);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "SetShowFilePropertiesDialog Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern void ShowFileConverter();
        public void pShowFileConverter()
        {
            try
            {
                ShowFileConverter();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "ShowFileConverter Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }



        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern Int64 WSQ_decode_stream(
            [In] byte[] input_wsq_compressed_byte_array,
            [In] [MarshalAs(UnmanagedType.I8)] Int64 input_wsq_compressed_byte_array_length,
            out IntPtr output_decompressed_byte_array,
            [MarshalAs(UnmanagedType.I8)] out Int64 width,
            [MarshalAs(UnmanagedType.I8)] out Int64 height,
            [MarshalAs(UnmanagedType.I8)] out Int64 ppi,
            out IntPtr comment_text);
        public Int64 pWSQ_decode_stream(
           [In] byte[] input_wsq_compressed_byte_array,
           [In] Int64 input_wsq_compressed_byte_array_length,
           out byte[] output_decompressed_byte_array,
           out Int64 width,
           out Int64 height,
           out Int64 ppi,
           out String comment_text)
        {
            Int64 retVal = -1;
            output_decompressed_byte_array = new byte[1];
            width = height = ppi = 0;
            comment_text = String.Empty; //need to initialize comment_text in order to avoid crashing

            try
            {
                WSQ_decode_stream(input_wsq_compressed_byte_array, input_wsq_compressed_byte_array_length,
                    out wsq_decompressed_byte_array_ptr, out width, out height,
                    out ppi, out out_comment_text_ptr);
                comment_text = Marshal.PtrToStringAnsi(out_comment_text_ptr);
                Int64 output_decompressed_byte_array_length = width * height;
                output_decompressed_byte_array = new byte[output_decompressed_byte_array_length];
                Marshal.Copy(wsq_decompressed_byte_array_ptr, output_decompressed_byte_array, 0, (Int32)output_decompressed_byte_array_length);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "WSQ_decode_stream Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return retVal;
        }
		
		
		
        [DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern Int64 WSQ_encode_stream(
            [In] byte[] input_wsq_decompressed_byte_array,
            [In] [MarshalAs(UnmanagedType.I8)] Int64 width,
            [In] [MarshalAs(UnmanagedType.I8)] Int64 height,
            [In] [MarshalAs(UnmanagedType.R8)] double bitrate,
            [In] [MarshalAs(UnmanagedType.I8)] Int64 ppi,
            [In] byte[] commentText,
            out IntPtr output_compressed_byte_array,
            [MarshalAs(UnmanagedType.I8)] out Int64 output_compressed_byte_array_length);
        public Int64 pWSQ_encode_stream(
            byte[] input_wsq_decompressed_byte_array,
            Int64 width,
            Int64 height,
            double bitrate,
            Int64 ppi,
            String commentText,
            out byte[] output_compressed_byte_array,
            out Int64 output_compressed_byte_array_length)
        {
            Int64 retVal = -1;
            output_compressed_byte_array = null;
            output_compressed_byte_array_length = 0;
            try
            {
                WSQ_encode_stream(input_wsq_decompressed_byte_array, width, height,
                    bitrate, ppi, utf8.GetBytes(commentText),
                    out wsq_compressed_byte_array_ptr, out output_compressed_byte_array_length);
                if (output_compressed_byte_array_length == 0)
                    throw new ArgumentException("output_compressed_byte_array_length == 0");
                output_compressed_byte_array = new byte[output_compressed_byte_array_length];
                Marshal.Copy(wsq_compressed_byte_array_ptr, output_compressed_byte_array, 0, (Int32)output_compressed_byte_array_length);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "WSQ_encode_stream Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return retVal;
        }






		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		private static extern IntPtr ConvertHBITMAPtoGrayScale256(IntPtr hBitmap);
		public IntPtr pConvertHBITMAPtoGrayScale256(IntPtr Bitmap)
		{
			try
			{
				bitmap_ptr = ConvertHBITMAPtoGrayScale256(Bitmap);
			}
			catch (Exception ex) 
			{
				MessageBox.Show(ex.Message, "ConvertHBITMAPtoGrayScale256 Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			return bitmap_ptr;
		}


		
		[DllImport("WSQ_library64.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
		private static extern Int64 SaveHBITMAPtoFileAsGrayScale256BMP(IntPtr hBitmap, byte[] filename);
		public Int64 pSaveHBITMAPtoFileAsGrayScale256BMP(IntPtr Bitmap, String FileName)
		{
			Int64 retVal = -1;
			if ((Bitmap == (IntPtr)null)||(FileName == null)) 
				throw new ArgumentNullException("Bitmap and FileName should be not null");
			if (FileName.Length == 0)
				throw new ArgumentException("Incorrect file name");
			try
			{
				retVal = SaveHBITMAPtoFileAsGrayScale256BMP(Bitmap, utf8.GetBytes(FileName));
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "SaveHBITMAPtoFileAsGrayScale256BMP Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			return retVal;
		}






    }
}
